@@ -17,8 +17,6 @@ static char yyrcsid[]
 #define YYRECOVERING() (yyerrflag!=0)
 #define YYPREFIX "yy"
 #line 23 "parse.y"
-#include "includes.h"
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -38,8 +36,7 @@ static struct ntpd_conf		*conf;
 static FILE			*fin = NULL;
 static int			 lineno = 1;
 static int			 errors = 0;
-static int			 pdebug = 1;
-char				*infile;
+const char			*infile;
 
 int	 yyerror(const char *, ...);
 int	 yyparse(void);
@@ -50,19 +47,6 @@ int	 lungetc(int);
 int	 findeol(void);
 int	 yylex(void);
 
-TAILQ_HEAD(symhead, sym)	 symhead = TAILQ_HEAD_INITIALIZER(symhead);
-struct sym {
-	TAILQ_ENTRY(sym)	 entries;
-	int			 used;
-	int			 persist;
-	char			*nam;
-	char			*val;
-};
-
-int	 symset(const char *, const char *, int);
-char	*symget(const char *);
-int	 atoul(char *, u_long *);
-
 typedef struct {
 	union {
 		u_int32_t		 number;
@@ -72,7 +56,7 @@ typedef struct {
 	int lineno;
 } YYSTYPE;
 
-#line 76 "y.tab.c"
+#line 60 "y.tab.c"
 #define LISTEN 257
 #define ON 258
 #define SERVER 259
@@ -86,8 +70,7 @@ const short yylhs[] =
 short yylhs[] =
 #endif
 	{                                        -1,
-    0,    0,    0,    0,    0,    1,    2,    2,    5,    4,
-    4,    4,    3,
+    0,    0,    0,    0,    2,    2,    2,    1,
 };
 #if defined(__cplusplus) || defined(__STDC__)
 const short yylen[] =
@@ -95,8 +78,7 @@ const short yylen[] =
 short yylen[] =
 #endif
 	{                                         2,
-    0,    2,    3,    3,    3,    1,    2,    1,    3,    3,
-    2,    2,    1,
+    0,    2,    3,    3,    3,    2,    2,    1,
 };
 #if defined(__cplusplus) || defined(__STDC__)
 const short yydefred[] =
@@ -104,9 +86,8 @@ const short yydefred[] =
 short yydefred[] =
 #endif
 	{                                      1,
-    0,    0,    0,    0,    0,    0,    2,    0,    0,    5,
-    0,   13,   12,   11,    0,    3,    4,   10,    8,    0,
-    7,
+    0,    0,    0,    0,    0,    2,    0,    4,    0,    8,
+    7,    6,    3,    5,
 };
 #if defined(__cplusplus) || defined(__STDC__)
 const short yydgoto[] =
@@ -114,7 +95,7 @@ const short yydgoto[] =
 short yydgoto[] =
 #endif
 	{                                       1,
-    0,   20,   13,    8,    9,
+   11,    7,
 };
 #if defined(__cplusplus) || defined(__STDC__)
 const short yysindex[] =
@@ -122,9 +103,8 @@ const short yysindex[] =
 short yysindex[] =
 #endif
 	{                                      0,
-  -10,   -8, -255, -258, -258,  -56,    0,   -2,   -1,    0,
- -258,    0,    0,    0, -256,    0,    0,    0,    0, -252,
-    0,};
+  -10,   -8, -255, -258, -258,    0,   -3,    0, -258,    0,
+    0,    0,    0,    0,};
 #if defined(__cplusplus) || defined(__STDC__)
 const short yyrindex[] =
 #else
@@ -132,25 +112,24 @@ short yyrindex[] =
 #endif
 	{                                      0,
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
-    0,    0,    0,    0,    0,    0,    0,    0,    0,    1,
-    0,};
+    0,    0,    0,    0,};
 #if defined(__cplusplus) || defined(__STDC__)
 const short yygindex[] =
 #else
 short yygindex[] =
 #endif
 	{                                      0,
-    0,    0,   -4,    0,    0,
+   -4,    0,
 };
-#define YYTABLESIZE 252
+#define YYTABLESIZE 250
 #if defined(__cplusplus) || defined(__STDC__)
 const short yytable[] =
 #else
 short yytable[] =
 #endif
-	{                                       7,
-   14,   10,   11,   12,   15,   19,   18,   16,   17,   21,
-    9,    0,    0,    0,    0,    0,    0,    0,    0,    0,
+	{                                       6,
+   12,    8,    9,   10,   14,    0,   13,    0,    0,    0,
+    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
@@ -174,7 +153,6 @@ short yytable[] =
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    2,    3,    0,    4,    5,
-    0,    6,
 };
 #if defined(__cplusplus) || defined(__STDC__)
 const short yycheck[] =
@@ -182,8 +160,8 @@ const short yycheck[] =
 short yycheck[] =
 #endif
 	{                                      10,
-    5,   10,  258,  262,   61,  262,   11,   10,   10,  262,
-   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
+    5,   10,  258,  262,    9,   -1,   10,   -1,   -1,   -1,
+   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
@@ -207,7 +185,6 @@ short yycheck[] =
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
    -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
    -1,   -1,   -1,   -1,   -1,  256,  257,   -1,  259,  260,
-   -1,  262,
 };
 #define YYFINAL 1
 #ifndef YYDEBUG
@@ -222,12 +199,12 @@ char *yyname[] =
 #endif
 	{
 "end-of-file",0,0,0,0,0,0,0,0,0,"'\\n'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'='",0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"LISTEN","ON","SERVER",
+0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"LISTEN","ON","SERVER",
 "SERVERS","ERROR","STRING",
 };
 #if defined(__cplusplus) || defined(__STDC__)
@@ -239,12 +216,7 @@ char *yyrule[] =
 "grammar :",
 "grammar : grammar '\\n'",
 "grammar : grammar conf_main '\\n'",
-"grammar : grammar varset '\\n'",
 "grammar : grammar error '\\n'",
-"number : STRING",
-"string : string STRING",
-"string : STRING",
-"varset : STRING '=' string",
 "conf_main : LISTEN ON address",
 "conf_main : SERVERS address",
 "conf_main : SERVER address",
@@ -275,7 +247,7 @@ short *yyss;
 short *yysslim;
 YYSTYPE *yyvs;
 int yystacksize;
-#line 239 "parse.y"
+#line 193 "parse.y"
 
 struct keywords {
 	const char	*k_name;
@@ -319,15 +291,10 @@ lookup(char *s)
 	p = bsearch(s, keywords, sizeof(keywords)/sizeof(keywords[0]),
 	    sizeof(keywords[0]), kw_cmp);
 
-	if (p) {
-		if (pdebug > 1)
-			fprintf(stderr, "%s: %d\n", s, p->k_val);
+	if (p)
 		return (p->k_val);
-	} else {
-		if (pdebug > 1)
-			fprintf(stderr, "string: %s\n", s);
+	else
 		return (STRING);
-	}
 }
 
 #define MAXPUSHBACK	128
@@ -420,11 +387,10 @@ int
 yylex(void)
 {
 	char	 buf[8096];
-	char	*p, *val;
+	char	*p;
 	int	 endc, c;
 	int	 token;
 
-top:
 	p = buf;
 	while ((c = lgetc(fin)) == ' ')
 		; /* nothing */
@@ -433,32 +399,6 @@ top:
 	if (c == '#')
 		while ((c = lgetc(fin)) != '\n' && c != EOF)
 			; /* nothing */
-	if (c == '$' && parsebuf == NULL) {
-		while (1) {
-			if ((c = lgetc(fin)) == EOF)
-				return (0);
-
-			if (p + 1 >= buf + sizeof(buf) - 1) {
-				yyerror("string too long");
-				return (findeol());
-			}
-			if (isalnum(c) || c == '_') {
-				*p++ = (char)c;
-				continue;
-			}
-			*p = '\0';
-			lungetc(c);
-			break;
-		}
-		val = symget(buf);
-		if (val == NULL) {
-			yyerror("macro \"%s\" not defined", buf);
-			return (findeol());
-		}
-		parsebuf = val;
-		parseindex = 0;
-		goto top;
-	}
 
 	switch (c) {
 	case '\'':
@@ -518,10 +458,8 @@ top:
 }
 
 int
-parse_config(char *filename, struct ntpd_conf *xconf)
+parse_config(const char *filename, struct ntpd_conf *xconf)
 {
-	struct sym		*sym, *next;
-
 	conf = xconf;
 	lineno = 1;
 	errors = 0;
@@ -538,113 +476,9 @@ parse_config(char *filename, struct ntpd_conf *xconf)
 
 	fclose(fin);
 
-	/* Free macros and check which have not been used. */
-	for (sym = TAILQ_FIRST(&symhead); sym != NULL; sym = next) {
-		next = TAILQ_NEXT(sym, entries);
-		if ((conf->opts & NTPD_OPT_VERBOSE2) && !sym->used)
-			fprintf(stderr, "warning: macro \"%s\" not "
-			    "used\n", sym->nam);
-		if (!sym->persist) {
-			free(sym->nam);
-			free(sym->val);
-			TAILQ_REMOVE(&symhead, sym, entries);
-			free(sym);
-		}
-	}
-
 	return (errors ? -1 : 0);
 }
-
-int
-symset(const char *nam, const char *val, int persist)
-{
-	struct sym	*sym;
-
-	for (sym = TAILQ_FIRST(&symhead); sym && strcmp(nam, sym->nam);
-	    sym = TAILQ_NEXT(sym, entries))
-		;	/* nothing */
-
-	if (sym != NULL) {
-		if (sym->persist == 1)
-			return (0);
-		else {
-			free(sym->nam);
-			free(sym->val);
-			TAILQ_REMOVE(&symhead, sym, entries);
-			free(sym);
-		}
-	}
-	if ((sym = calloc(1, sizeof(*sym))) == NULL)
-		return (-1);
-
-	sym->nam = strdup(nam);
-	if (sym->nam == NULL) {
-		free(sym);
-		return (-1);
-	}
-	sym->val = strdup(val);
-	if (sym->val == NULL) {
-		free(sym->nam);
-		free(sym);
-		return (-1);
-	}
-	sym->used = 0;
-	sym->persist = persist;
-	TAILQ_INSERT_TAIL(&symhead, sym, entries);
-	return (0);
-}
-
-int
-cmdline_symset(char *s)
-{
-	char	*sym, *val;
-	int	ret;
-	size_t	len;
-
-	if ((val = strrchr(s, '=')) == NULL)
-		return (-1);
-
-	len = strlen(s) - strlen(val) + 1;
-	if ((sym = malloc(len)) == NULL)
-		fatal("cmdline_symset: malloc");
-
-	strlcpy(sym, s, len);
-
-	ret = symset(sym, val + 1, 1);
-	free(sym);
-
-	return (ret);
-}
-
-char *
-symget(const char *nam)
-{
-	struct sym	*sym;
-
-	TAILQ_FOREACH(sym, &symhead, entries)
-		if (strcmp(nam, sym->nam) == 0) {
-			sym->used = 1;
-			return (sym->val);
-		}
-	return (NULL);
-}
-
-int
-atoul(char *s, u_long *ulvalp)
-{
-	u_long	 ulval;
-	char	*ep;
-
-	errno = 0;
-	ulval = strtoul(s, &ep, 0);
-	if (s[0] == '\0' || *ep != '\0')
-		return (-1);
-	if (errno == ERANGE && ulval == ULONG_MAX)
-		return (-1);
-	*ulvalp = ulval;
-	return (0);
-}
-#line 596 "y.tab.c"
+#line 430 "y.tab.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
 #if defined(__cplusplus) || defined(__STDC__)
 static int yygrowstack(void)
@@ -837,58 +671,25 @@ yyreduce:
     yyval = yyvsp[1-yym];
     switch (yyn)
     {
-case 5:
-#line 93 "parse.y"
+case 4:
+#line 74 "parse.y"
 { errors++; }
 break;
-case 6:
-#line 96 "parse.y"
-{
-			u_long	ulval;
-
-			if (atoul(yyvsp[0].v.string, &ulval) == -1) {
-				yyerror("\"%s\" is not a number", yyvsp[0].v.string);
-				free(yyvsp[0].v.string);
-				YYERROR;
-			} else
-				yyval.v.number = ulval;
-			free(yyvsp[0].v.string);
-		}
-break;
-case 7:
-#line 109 "parse.y"
-{
-			if (asprintf(&yyval.v.string, "%s %s", yyvsp[-1].v.string, yyvsp[0].v.string) == -1)
-				fatal("string: asprintf");
-			free(yyvsp[-1].v.string);
-			free(yyvsp[0].v.string);
-		}
-break;
-case 9:
-#line 118 "parse.y"
-{
-			if (conf->opts & NTPD_OPT_VERBOSE)
-				printf("%s = \"%s\"\n", yyvsp[-2].v.string, yyvsp[0].v.string);
-			if (symset(yyvsp[-2].v.string, yyvsp[0].v.string, 0) == -1)
-				fatal("cannot store variable");
-			free(yyvsp[-2].v.string);
-			free(yyvsp[0].v.string);
-		}
-break;
-case 10:
-#line 128 "parse.y"
+case 5:
+#line 77 "parse.y"
 {
 			struct listen_addr	*la;
 			struct ntp_addr		*h, *next;
 
-			if (yyvsp[0].v.addr->a == NULL) {
-				yyerror("cannot resolve \"%s\"", yyvsp[0].v.addr->name);
+			if ((h = yyvsp[0].v.addr->a) == NULL &&
+			    (host_dns(yyvsp[0].v.addr->name, &h) == -1 || !h)) {
+				yyerror("could not resolve \"%s\"", yyvsp[0].v.addr->name);
 				free(yyvsp[0].v.addr->name);
 				free(yyvsp[0].v.addr);
 				YYERROR;
 			}
 
-			for (h = yyvsp[0].v.addr->a; h != NULL; h = next) {
+			for (; h != NULL; h = next) {
 				next = h->next;
 				if (h->ss.ss_family == AF_UNSPEC) {
 					conf->listen_all = 1;
@@ -909,8 +710,8 @@ case 10:
 			free(yyvsp[0].v.addr);
 		}
 break;
-case 11:
-#line 159 "parse.y"
+case 6:
+#line 109 "parse.y"
 {
 			struct ntp_peer		*p;
 			struct ntp_addr		*h, *next;
@@ -923,6 +724,7 @@ case 11:
 					    h->ss.ss_family != AF_INET6) {
 						yyerror("IPv4 or IPv6 address "
 						    "or hostname expected");
+						free(h);
 						free(yyvsp[0].v.addr->name);
 						free(yyvsp[0].v.addr);
 						YYERROR;
@@ -947,8 +749,8 @@ case 11:
 			free(yyvsp[0].v.addr);
 		}
 break;
-case 12:
-#line 194 "parse.y"
+case 7:
+#line 145 "parse.y"
 {
 			struct ntp_peer		*p;
 			struct ntp_addr		*h, *next;
@@ -960,6 +762,8 @@ case 12:
 				    h->ss.ss_family != AF_INET6) {
 					yyerror("IPv4 or IPv6 address "
 					    "or hostname expected");
+					free(h);
+					free(p);
 					free(yyvsp[0].v.addr->name);
 					free(yyvsp[0].v.addr);
 					YYERROR;
@@ -978,8 +782,8 @@ case 12:
 			free(yyvsp[0].v.addr);
 		}
 break;
-case 13:
-#line 224 "parse.y"
+case 8:
+#line 177 "parse.y"
 {
 			if ((yyval.v.addr = calloc(1, sizeof(struct ntp_addr_wrap))) ==
 			    NULL)
@@ -988,12 +792,13 @@ case 13:
 				yyerror("could not parse address spec \"%s\"",
 				    yyvsp[0].v.string);
 				free(yyvsp[0].v.string);
+				free(yyval.v.addr);
 				YYERROR;
 			}
 			yyval.v.addr->name = yyvsp[0].v.string;
 		}
 break;
-#line 945 "y.tab.c"
+#line 750 "y.tab.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
